#ifndef Record_h
#define Record_h


namespace fastmrc
{

// MARC 21 Specifications for Record Structure: http://www.loc.gov/marc/specifications/specrecstruc.html
class DECLSPEC Record
{
	friend std::istream& operator>> (std::istream& is, Record& rec)
	{
		Record::readRec(is, rec);
		return is;
	}
	

	typedef std::multimap<FieldTag, Field*> FieldMap;
		
	static void readRec(std::istream& is, Record& rec);
	Record(const Record&);
	Record& operator=(const Record&);
public:
	typedef FieldMap::const_iterator FieldMapCIt;
	typedef std::pair<FieldMapCIt, FieldMapCIt> FieldMapRange;

	Record(void);
	Record(const char* _buf, unsigned _bufsize); // Create record from buffer

	virtual ~Record(void);
	int getId();
	void getMD5(unsigned char digest[16]) const;

	const char* getLeader() const;

	int getLeaderLength() const;

	FieldMapCIt getFMapBegin();

	FieldMapCIt getFMapEnd();

	FieldMapCIt getFirstField(const char* tag);

	FieldMapRange getFieldRange(const char* tag);

	const char* getData() const;

	unsigned getDatalen() const;
protected:
	void clean(void);
	void readId(void);
	void readFields();
private:
	char* buf; // unchecked pointer to record buffer
	unsigned bufsize; // size of record buffer
	unsigned reclen; // length of record (reclen <= bufsize)
	int id;
	unsigned baseOffset;
	FieldMap *fmap;
};


} // namespace fastmrc


#endif //#define Record_h